@@ -1,16 +1,16 @@
 #' Calculation of integral-based distance- and angle-based 3D imbalance indices
 #'
-#' \code{all3DImbalIndices} - This serves as a wrapper function to calculate 
-#' a 3D imbalance index value of a 3D tree in phylo3D format according to the 
+#' \code{all3DImbalIndices} - This serves as a wrapper function to calculate
+#' a 3D imbalance index value of a 3D tree in phylo3D format according to the
 #' specified imbalance measurement and weighting scheme.\cr
-#' If problems occur with the estimation of the integrals, try to increase 
+#' If problems occur with the estimation of the integrals, try to increase
 #' the optional parameters  \cr\code{rel.tolerance} and \code{max.subdiv}.
-#' 
+#'
 #' @author Sophie Kersting
-#' 
-#' @param tree A rooted tree in phylo3D format (no special node enumeration 
+#'
+#' @param tree A rooted tree in phylo3D format (no special node enumeration
 #' required, except that nodes are numbered from 1 to |V| = the total number of
-#' nodes). There must be at least 2 nodes, i.e., one edge. The attributes 
+#' nodes). There must be at least 2 nodes, i.e., one edge. The attributes
 #' 'node.coord' and 'edge.weight' are strictly required.
 #' @param imbal_type Specifies which node imbalance measurement should be used.
 #' Available are:\cr
@@ -20,63 +20,72 @@
 #' "mu"    - relative centroid distance
 #' @param weight Specifies how the node imbalance values should be weighted.
 #' Available weighting methods are: \cr
-#' "edge_weight" (default) -> Imbalance with regards to the total edge 
+#' "edge_weight" (default) -> Imbalance with regards to the total edge
 #' weight.\cr
 #' "edge_length" -> Imbalance with regards to the total edge length.
-#' @param rel.tolerance (Optional) Numeric value which specifies the relative 
-#' tolerance which should be used for estimating the integral using 
-#' stats::integrate. Set to 1e-10 by default (the stats::integrate default 
+#' @param rel.tolerance (Optional) Numeric value which specifies the relative
+#' tolerance which should be used for estimating the integral using
+#' stats::integrate. Set to 1e-10 by default (the stats::integrate default
 #' value is approx 3e-16).
-#' @param max.subdiv (Optional) Integer value which specifies the maximal number 
-#' of interval subdivisions for estimating the integral using stats::integrate. 
+#' @param max.subdiv (Optional) Integer value which specifies the maximal number
+#' of interval subdivisions for estimating the integral using stats::integrate.
 #' Set to 200 by default (stats::integrate default value 100).
-#' 
+#'
 #' @return \code{all3DImbalIndices} Numeric value indicating the internal 3D
 #' imbalance according to the chosen method.
 #'
 #' @export
 #' @rdname integralImbalIndices
-#' 
+#'
 #' @examples
 #' tree <- treeDbalance::extendPhylo(treeDbalance::example3Dtrees$bean09)
-#' all3DImbalIndices(tree, imbal_type = "A", weight="edge_length")
-all3DImbalIndices <- function(tree, imbal_type, weight = "edge_weight", 
-                  rel.tolerance = 1e-10, max.subdiv = 200L){
+#' all3DImbalIndices(tree, imbal_type = "A", weight = "edge_length")
+all3DImbalIndices <- function(tree, imbal_type, weight = "edge_weight",
+                              rel.tolerance = 1e-10, max.subdiv = 200L) {
   if (imbal_type == "mu") {
-    mu_Index(tree = tree, weight = weight, 
-             rel.tolerance = rel.tolerance, max.subdiv = max.subdiv)
+    mu_Index(
+      tree = tree, weight = weight,
+      rel.tolerance = rel.tolerance, max.subdiv = max.subdiv
+    )
   } else if (imbal_type == "M") {
-    M_Index(tree = tree, weight = weight, 
-             rel.tolerance = rel.tolerance, max.subdiv = max.subdiv)
+    M_Index(
+      tree = tree, weight = weight,
+      rel.tolerance = rel.tolerance, max.subdiv = max.subdiv
+    )
   } else if (imbal_type == "A") {
-    A_Index(tree = tree, weight = weight, 
-             rel.tolerance = rel.tolerance, max.subdiv = max.subdiv)
+    A_Index(
+      tree = tree, weight = weight,
+      rel.tolerance = rel.tolerance, max.subdiv = max.subdiv
+    )
   } else if (imbal_type == "alpha") {
-    alpha_Index(tree = tree, weight = weight, 
-             rel.tolerance = rel.tolerance, max.subdiv = max.subdiv)
+    alpha_Index(
+      tree = tree, weight = weight,
+      rel.tolerance = rel.tolerance, max.subdiv = max.subdiv
+    )
   } else {
     stop(paste("Unknown node imbalance measurement '", imbal_type,
-               "'. Available are: 'A', 'alpha', 'M' and 'mu'.",
-               sep = ""))
+      "'. Available are: 'A', 'alpha', 'M' and 'mu'.",
+      sep = ""
+    ))
   }
 }
 #' Calculation of integral-based distance- and angle-based 3D imbalance indices
 #'
-#' \code{A_Index} - Calculates the 3D imbalance index "weighted integral-based 
-#' centroid angle" of a 3D tree in phylo3D format using either the 
+#' \code{A_Index} - Calculates the 3D imbalance index "weighted integral-based
+#' centroid angle" of a 3D tree in phylo3D format using either the
 #' edge weights or the edge lengths as weights.\cr
-#' If problems occur with the estimation of the integrals, try to increase 
+#' If problems occur with the estimation of the integrals, try to increase
 #' the optional parameters  \cr\code{rel.tolerance} and \code{max.subdiv}.
 #'
-#' @return \code{A_Index} Numeric value in the interval 
-#' between 0 (included) and \eqn{\pi} (excluded). A value near \eqn{\pi} 
+#' @return \code{A_Index} Numeric value in the interval
+#' between 0 (included) and \eqn{\pi} (excluded). A value near \eqn{\pi}
 #' indicates a higher degree and near 0 a lower degree of asymmetry.
 #'
 #' @export
 #' @rdname integralImbalIndices
-#' 
+#'
 #' @examples
-#' A_Index(tree, weight="edge_weight")
+#' A_Index(tree, weight = "edge_weight")
 A_Index <- function(tree, weight = "edge_weight", rel.tolerance = 1e-10,
                     max.subdiv = 200L) {
   if (!inherits(tree, "phylo") && !inherits(tree, "phylo3D")) {
@@ -85,20 +94,23 @@ A_Index <- function(tree, weight = "edge_weight", rel.tolerance = 1e-10,
   if (!weight %in% c("edge_weight", "edge_length")) {
     warning(paste(
       "Unknown weighting method. The default 'edge_weight' is used",
-      "instead."))
+      "instead."
+    ))
     weight <- "edge_weight"
   }
   if (sum(c("node.coord", "edge.weight") %in% attributes(tree)$names) != 2) {
     stop(paste(
       "Cannot calculate subtree centroids. The tree is missing at least one",
-      "of these attributes:'node.coord', 'edge.weight'."))
+      "of these attributes:'node.coord', 'edge.weight'."
+    ))
   }
   if (sum(c("node.descs", "node.ancs", "node.depth", "node.subtrCentr") %in%
-          attributes(tree)$names) < 4) {
-    comment(paste(
+    attributes(tree)$names) < 4) {
+    message(paste(
       "This may take longer as at least one of the attributes,",
       "'node.descs', 'node.ancs',", "'node.depth' and 'node.subtrCentr'",
-      "does not exist and has to be calculated first."))
+      "does not exist and has to be calculated first."
+    ))
   }
   if (!"node.descs" %in% attributes(tree)$names) {
     tree$node.descs <- getDescs(tree)
@@ -120,9 +132,11 @@ A_Index <- function(tree, weight = "edge_weight", rel.tolerance = 1e-10,
     wei_v <- inc_edgeweights
   } else {
     wei_v <- getIncEdgeLens(tree)
-    if(sum(inc_edgeweights[node_order[-1]]==0)>0){
-      warning(paste("It might not be advisable to use edge_length as",
-                    "weighting method if there are edges with zero weight.\n"))
+    if (sum(inc_edgeweights[node_order[-1]] == 0) > 0) {
+      warning(paste(
+        "It might not be advisable to use edge_length as",
+        "weighting method if there are edges with zero weight.\n"
+      ))
     }
   }
   #-----------
@@ -158,19 +172,19 @@ A_Index <- function(tree, weight = "edge_weight", rel.tolerance = 1e-10,
 }
 #' Calculation of integral-based distance- and angle-based 3D imbalance indices
 #'
-#' \code{alpha_Index} - Calculates the 3D imbalance index "weighted 
-#' integral-based minimal centroid angle" of a 3D tree in phylo3D format using 
+#' \code{alpha_Index} - Calculates the 3D imbalance index "weighted
+#' integral-based minimal centroid angle" of a 3D tree in phylo3D format using
 #' either the edge weights or the edge lengths as weights.\cr
-#' If problems occur with the estimation of the integrals, try to increase 
+#' If problems occur with the estimation of the integrals, try to increase
 #' the optional parameters  \cr\code{rel.tolerance} and \code{max.subdiv}.
 #'
-#' @return \code{alpha_Index} Numeric value in the interval 
-#' between 0 (included) and \eqn{\pi/2} (excluded). A value near \eqn{\pi/2} 
+#' @return \code{alpha_Index} Numeric value in the interval
+#' between 0 (included) and \eqn{\pi/2} (excluded). A value near \eqn{\pi/2}
 #' indicates a higher degree and near 0 a lower degree of asymmetry.
 #'
 #' @export
 #' @rdname integralImbalIndices
-#' 
+#'
 #' @examples
 #' alpha_Index(tree)
 alpha_Index <- function(tree, weight = "edge_weight", rel.tolerance = 1e-10,
@@ -181,21 +195,24 @@ alpha_Index <- function(tree, weight = "edge_weight", rel.tolerance = 1e-10,
   if (!weight %in% c("edge_weight", "edge_length")) {
     warning(paste(
       "Unknown weighting method. The default 'edge_weight' is used",
-      "instead."))
+      "instead."
+    ))
     weight <- "edge_weight"
   }
   if (sum(c("node.coord", "edge.weight") %in% attributes(tree)$names) != 2) {
     stop(paste(
       "Cannot calculate subtree centroids. The tree is missing at least one",
-      "of these attributes:'node.coord', 'edge.weight'."))
+      "of these attributes:'node.coord', 'edge.weight'."
+    ))
   }
   if (sum(c("node.descs", "node.ancs", "node.depth", "node.subtrCentr") %in%
-          attributes(tree)$names) < 4) {
-    comment(paste(
+    attributes(tree)$names) < 4) {
+    message(paste(
       "This may take longer as at least one of the attributes,",
       "'node.descs', 'node.ancs',",
       "'node.depth' and 'node.subtrCentr' does not",
-      "exist and has to be calculated first."))
+      "exist and has to be calculated first."
+    ))
   }
   if (!"node.descs" %in% attributes(tree)$names) {
     tree$node.descs <- getDescs(tree)
@@ -217,9 +234,11 @@ alpha_Index <- function(tree, weight = "edge_weight", rel.tolerance = 1e-10,
     wei_v <- inc_edgeweights
   } else {
     wei_v <- getIncEdgeLens(tree)
-    if(sum(inc_edgeweights[node_order[-1]]==0)>0){
-      warning(paste("It might not be advisable to use edge_length as",
-                    "weighting method if there are edges with zero weight.\n"))
+    if (sum(inc_edgeweights[node_order[-1]] == 0) > 0) {
+      warning(paste(
+        "It might not be advisable to use edge_length as",
+        "weighting method if there are edges with zero weight.\n"
+      ))
     }
   }
   #-----------
@@ -255,44 +274,47 @@ alpha_Index <- function(tree, weight = "edge_weight", rel.tolerance = 1e-10,
 }
 #' Calculation of integral-based distance- and angle-based 3D imbalance indices
 #'
-#' \code{M_Index} - Calculates the 3D imbalance index "weighted integral-based 
-#' expanded relative centroid distance" of a 3D tree in phylo3D format using 
+#' \code{M_Index} - Calculates the 3D imbalance index "weighted integral-based
+#' expanded relative centroid distance" of a 3D tree in phylo3D format using
 #' either the edge weights or the edge lengths as weights.\cr
-#' If problems occur with the estimation of the integrals, try to increase 
+#' If problems occur with the estimation of the integrals, try to increase
 #' the optional parameters  \cr\code{rel.tolerance} and \code{max.subdiv}.
-#' 
-#' @return \code{M_Index} Numeric value in the interval 
-#' between 0 (included) and 1 (excluded). A value near 1 indicates a higher 
+#'
+#' @return \code{M_Index} Numeric value in the interval
+#' between 0 (included) and 1 (excluded). A value near 1 indicates a higher
 #' degree and near 0 a lower degree of asymmetry.
 #'
 #' @export
 #' @rdname integralImbalIndices
-#' 
+#'
 #' @examples
-#' M_Index(tree, weight="edge_length")
+#' M_Index(tree, weight = "edge_length")
 M_Index <- function(tree, weight = "edge_weight", rel.tolerance = 1e-10,
-                  max.subdiv = 200L) {
+                    max.subdiv = 200L) {
   if (!inherits(tree, "phylo") && !inherits(tree, "phylo3D")) {
     stop("The input tree must have class phylo or phylo3D.")
   }
   if (!weight %in% c("edge_weight", "edge_length")) {
     warning(paste(
       "Unknown weighting method. The default 'edge_weight' is used",
-      "instead."))
+      "instead."
+    ))
     weight <- "edge_weight"
   }
   if (sum(c("node.coord", "edge.weight") %in% attributes(tree)$names) != 2) {
     stop(paste(
       "Cannot calculate subtree centroids. The tree is missing at least one",
-      "of these attributes:'node.coord', 'edge.weight'."))
+      "of these attributes:'node.coord', 'edge.weight'."
+    ))
   }
   if (sum(c("node.descs", "node.ancs", "node.depth", "node.subtrCentr") %in%
-          attributes(tree)$names) < 4) {
-    comment(paste(
+    attributes(tree)$names) < 4) {
+    message(paste(
       "This may take longer as at least one of the attributes,",
       "'node.descs', 'node.ancs',",
       "'node.depth' and 'node.subtrCentr' does not",
-      "exist and has to be calculated first."))
+      "exist and has to be calculated first."
+    ))
   }
   if (!"node.descs" %in% attributes(tree)$names) {
     tree$node.descs <- getDescs(tree)
@@ -314,9 +336,11 @@ M_Index <- function(tree, weight = "edge_weight", rel.tolerance = 1e-10,
     wei_v <- inc_edgeweights
   } else {
     wei_v <- getIncEdgeLens(tree)
-    if(sum(inc_edgeweights[node_order[-1]]==0)>0){
-      warning(paste("It might not be advisable to use edge_length as",
-                    "weighting method if there are edges with zero weight.\n"))
+    if (sum(inc_edgeweights[node_order[-1]] == 0) > 0) {
+      warning(paste(
+        "It might not be advisable to use edge_length as",
+        "weighting method if there are edges with zero weight.\n"
+      ))
     }
   }
   #-----------
@@ -353,21 +377,21 @@ M_Index <- function(tree, weight = "edge_weight", rel.tolerance = 1e-10,
 }
 #' Calculation of integral-based distance- and angle-based 3D imbalance indices
 #'
-#' \code{mu_Int} - Calculates the 3D imbalance index "weighted integral-based 
-#' relative centroid distance" of a 3D tree in phylo3D format using either the 
+#' \code{mu_Int} - Calculates the 3D imbalance index "weighted integral-based
+#' relative centroid distance" of a 3D tree in phylo3D format using either the
 #' edge weights or the edge lengths as weights.\cr
-#' If problems occur with the estimation of the integrals, try to increase 
-#' the optional parameters  \cr\code{rel.tolerance} and \code{max.subdiv}. 
-#' 
-#' @return \code{mu_Index} Numeric value in the interval between 0 (included) 
-#' and 1 (excluded). A value near 1 indicates a higher degree and near 0 a 
+#' If problems occur with the estimation of the integrals, try to increase
+#' the optional parameters  \cr\code{rel.tolerance} and \code{max.subdiv}.
+#'
+#' @return \code{mu_Index} Numeric value in the interval between 0 (included)
+#' and 1 (excluded). A value near 1 indicates a higher degree and near 0 a
 #' lower degree of asymmetry.
 #'
 #' @export
 #' @rdname integralImbalIndices
-#' 
+#'
 #' @examples
-#' mu_Index(tree, weight="edge_length")
+#' mu_Index(tree, weight = "edge_length")
 mu_Index <- function(tree, weight = "edge_weight", rel.tolerance = 1e-10,
                      max.subdiv = 200L) {
   if (!inherits(tree, "phylo") && !inherits(tree, "phylo3D")) {
@@ -376,21 +400,24 @@ mu_Index <- function(tree, weight = "edge_weight", rel.tolerance = 1e-10,
   if (!weight %in% c("edge_weight", "edge_length")) {
     warning(paste(
       "Unknown weighting method. The default 'edge_weight' is used",
-      "instead."))
+      "instead."
+    ))
     weight <- "edge_weight"
   }
   if (sum(c("node.coord", "edge.weight") %in% attributes(tree)$names) != 2) {
     stop(paste(
       "Cannot calculate subtree centroids. The tree is missing at least one",
-      "of these attributes:'node.coord', 'edge.weight'."))
+      "of these attributes:'node.coord', 'edge.weight'."
+    ))
   }
   if (sum(c("node.descs", "node.ancs", "node.depth", "node.subtrCentr") %in%
-          attributes(tree)$names) < 4) {
-    comment(paste(
+    attributes(tree)$names) < 4) {
+    message(paste(
       "This may take longer as at least one of the attributes,",
       "'node.descs', 'node.ancs',",
       "'node.depth' and 'node.subtrCentr' does not",
-      "exist and has to be calculated first."))
+      "exist and has to be calculated first."
+    ))
   }
   if (!"node.descs" %in% attributes(tree)$names) {
     tree$node.descs <- getDescs(tree)
@@ -412,9 +439,11 @@ mu_Index <- function(tree, weight = "edge_weight", rel.tolerance = 1e-10,
     wei_v <- inc_edgeweights
   } else {
     wei_v <- getIncEdgeLens(tree)
-    if(sum(inc_edgeweights[node_order[-1]]==0)>0){
-      warning(paste("It might not be advisable to use edge_length as",
-                    "weighting method if there are edges with zero weight.\n"))
+    if (sum(inc_edgeweights[node_order[-1]] == 0) > 0) {
+      warning(paste(
+        "It might not be advisable to use edge_length as",
+        "weighting method if there are edges with zero weight.\n"
+      ))
     }
   }
   #-----------
@@ -457,38 +486,60 @@ mu_Index <- function(tree, weight = "edge_weight", rel.tolerance = 1e-10,
 #' @param p Numeric vector of size 3 (3D coordinates of parent node \eqn{p}).
 #' @param centr_v Numeric vector of size 3 (3D coordinates of the centroid of
 #' the pending subtree of node \eqn{v}).
-#' @param centr_v_weight Numeric value >=0 (weight of the pending subtree of 
+#' @param centr_v_weight Numeric value >=0 (weight of the pending subtree of
 #' node \eqn{v}).
 #' @param edge_weight Numeric value >=0 (weight of the edge \eqn{(p,v)}).
 #'
 #' @return \code{imbalInt_e} Numeric value (0 minimal value, higher values
-#'indicate a higher degree of asymmetry).
+#' indicate a higher degree of asymmetry).
 #'
 #' @export
 #' @rdname integralImbalIndices
 #'
 #' @examples
-#' imbalInt_e(p=c(0,0,1),v=c(0,0,0),centr_v=c(0.5,0,0),
-#'     centr_v_weight=1,edge_weight=1,imbal_type="mu")
+#' imbalInt_e(
+#'   p = c(0, 0, 1), v = c(0, 0, 0), centr_v = c(0.5, 0, 0),
+#'   centr_v_weight = 1, edge_weight = 1, imbal_type = "mu"
+#' )
 imbalInt_e <- function(p, v, centr_v, centr_v_weight, edge_weight,
                        imbal_type, rel.tolerance = 1e-8, max.subdiv = 200L) {
-  if (sum(is.na(c(p, v, centr_v, centr_v_weight, edge_weight))) > 0 ||
-      length(p) != 3 || length(v) != 3 || length(centr_v) != 3 ||
-      length(centr_v_weight) != 1 || length(edge_weight) != 1 ||
-      sum(p == v) == 3 ||
-      edge_weight < 0 || centr_v_weight < 0 || (edge_weight+centr_v_weight)==0){
-    stop(paste("Invalid input",
-               ": p=", paste(p, collapse = " "),
-               "; v=", paste(v, collapse = " "),
-               "; centr_v=", paste(centr_v, collapse = " "),
-               "; centr_v_weight=", paste(centr_v_weight, collapse = " "),
-               "; edge_weight=", paste(edge_weight, collapse = " "),
-               sep = ""))
+  inval_input_message <- paste("Input: ",
+    "p = c(", paste(p, collapse = ", "),
+    "), v = c(", paste(v, collapse = ", "),
+    "), centr_v = c(", paste(centr_v, collapse = ", "),
+    "), centr_v_weight = ", paste(centr_v_weight, collapse = ", "),
+    ", edge_weight = ", paste(edge_weight, collapse = ", "),
+    sep = ""
+  )
+  if (sum(is.na(c(p, v, centr_v, centr_v_weight, edge_weight))) > 0) {
+    stop(paste("There are NAs.", inval_input_message))
+  }
+  if (length(p) != 3 || length(v) != 3 || length(centr_v) != 3 ||
+    length(centr_v_weight) != 1 || length(edge_weight) != 1) {
+    stop(paste("The input has not the correct length.", inval_input_message))
+  }
+  if (edge_weight < 0 || centr_v_weight < 0) {
+    stop(paste("Negative weights.", inval_input_message))
   }
   if (!imbal_type %in% c("mu", "M", "A", "alpha")) {
     stop(paste("Unknown node imbalance measurement '", imbal_type,
-               "'. Available are: 'A', 'alpha', 'M' and 'mu'.",
-               sep = ""))
+      "'. Available are: 'A', 'alpha', 'M' and 'mu'.",
+      sep = ""
+    ))
+  }
+  if (sum(p == v) == 3) {
+    warning(paste(
+      "Parent and child have the same coordinates.",
+      "Returning zero instead.", inval_input_message
+    ))
+    return(0)
+  }
+  if (edge_weight == 0 && centr_v_weight == 0) {
+    warning(paste(
+      "All weights are zero. Returning zero.",
+      inval_input_message
+    ))
+    return(0)
   }
   node_imbal_fun <- function(x) {
     imbalProfile_e(
@@ -498,53 +549,39 @@ imbalInt_e <- function(p, v, centr_v, centr_v_weight, edge_weight,
   }
   result <- tryCatch(
     stats::integrate(node_imbal_fun,
-                     lower = 0, upper = 1,
-                     subdivisions = max.subdiv, rel.tol = rel.tolerance
+      lower = 0, upper = 1,
+      subdivisions = max.subdiv, rel.tol = rel.tolerance
     ),
     error = function(cond) {
-      message(paste("stats::integrate has severe troubles integrating. Input",
-                    ": p=", paste(p, collapse = " "),
-                    "; v=", paste(v, collapse = " "),
-                    "; centr_v=", paste(centr_v, collapse = " "),
-                    "; centr_v_weight=", paste(centr_v_weight, collapse = " "),
-                    "; edge_weight=", paste(edge_weight, collapse = " "),
-                    "; imbal_type=", paste(imbal_type, collapse = " "),
-                    "; rel.tolerance=", paste(rel.tolerance, collapse = " "),
-                    "; max.subdiv=", paste(max.subdiv, collapse = " "),
-                    ". This mostly occurs for integrals that are approx. zero.",
-                    "Thus, zero is returned.",
-                    sep = ""))
-      message("The original error message:")
-      message(cond)
-      return(0)
+      message(paste("stats::integrate has severe troubles integrating. ",
+        "This mostly occurs for integrals that are approximately zero. ",
+        "Thus, zero is returned.",
+        inval_input_message,
+        sep = ""
+      ))
+
+      print(utils::str(cond))
+      message("The original error message while integrating:")
+      message(conditionMessage(cond))
+      return(list(value = 0))
     },
     warning = function(cond) {
-      message(paste("stats::integrate has some troubles integrating. Input",
-                    ": p=", paste(p, collapse = " "),
-                    "; v=", paste(v, collapse = " "),
-                    "; centr_v=", paste(centr_v, collapse = " "),
-                    "; centr_v_weight=", paste(centr_v_weight, collapse = " "),
-                    "; edge_weight=", paste(edge_weight, collapse = " "),
-                    "; imbal_type=", paste(imbal_type, collapse = " "),
-                    "; rel.tolerance=", paste(rel.tolerance, collapse = " "),
-                    "; max.subdiv=", paste(max.subdiv, collapse = " "),
-                    ". This mostly occurs for integrals that are approx. zero.",
-                    "Thus, zero is returned.",
-                    sep = ""))
+      message(paste("stats::integrate has some troubles integrating. ",
+        "This mostly occurs for integrals that are approximately zero. ",
+        "Thus, zero is returned.",
+        inval_input_message,
+        sep = ""
+      ))
       message("The original warning message:")
-      message(cond)
-      return(0)
+      message(conditionMessage(cond))
+      return(list(value = 0))
     }
   )
   if (is.na(result$value)) {
-    stop(paste("A problem occured when trying to integrate using this input",
-               ": p=", paste(p, collapse = " "),
-               "; v=", paste(v, collapse = " "),
-               "; centr_v=", paste(centr_v, collapse = " "),
-               "; centr_v_weight=", paste(centr_v_weight, collapse = " "),
-               "; edge_weight=", paste(edge_weight, collapse = " "),
-               "; imbal_type=", paste(imbal_type, collapse = " "),
-               sep = ""))
+    stop(paste("A problem occured when trying to integrate the following ",
+      inval_input_message,
+      sep = ""
+    ))
   } else {
     return(result$value)
   }
@@ -557,7 +594,7 @@ imbalInt_e <- function(p, v, centr_v, centr_v_weight, edge_weight,
 #' @param xs Numeric vector with values between 0 (included) and 1 (excluded).
 #' Set of edge subdivisions.
 #'
-#' @return \code{imbalProfile_e} Numeric vector of imbalance values (0 minimal 
+#' @return \code{imbalProfile_e} Numeric vector of imbalance values (0 minimal
 #' value, higher values indicate a higher degree of asymmetry) for the edge
 #' subdivisions indicated by input \code{xs}.
 #'
@@ -565,23 +602,47 @@ imbalInt_e <- function(p, v, centr_v, centr_v_weight, edge_weight,
 #' @rdname integralImbalIndices
 #'
 #' @examples
-#' imbalProfile_e(xs=c(0,0.2,0.4),p=c(1,1,0),v=c(0,0,0),centr_v=c(0.5,0,0),
-#'     centr_v_weight=1,edge_weight=1,imbal_type="A")
+#' imbalProfile_e(
+#'   xs = c(0, 0.2, 0.4), p = c(1, 1, 0), v = c(0, 0, 0), centr_v = c(0.5, 0, 0),
+#'   centr_v_weight = 1, edge_weight = 1, imbal_type = "A"
+#' )
 imbalProfile_e <- function(xs, p, v, centr_v, centr_v_weight, edge_weight,
                            imbal_type) {
-  if (sum(is.na(c(p, v, centr_v, centr_v_weight, edge_weight))) > 0 ||
-      length(p) != 3 || length(v) != 3 || length(centr_v) != 3 ||
-      length(centr_v_weight) != 1 || length(edge_weight) != 1 ||
-      sum(p == v) == 3 || min(xs) < 0 || max(xs) >= 1 ||
-      edge_weight < 0 || centr_v_weight < 0 || (edge_weight+centr_v_weight)==0){
-    stop(paste("Invalid input",
-               ": xs=", paste(xs, collapse = " "),
-               "; p=", paste(p, collapse = " "),
-               "; v=", paste(v, collapse = " "),
-               "; centr_v=", paste(centr_v, collapse = " "),
-               "; centr_v_weight=", paste(centr_v_weight, collapse = " "),
-               "; edge_weight=", paste(edge_weight, collapse = " "),
-               sep = ""))
+  inval_input_message <- paste("Input: ",
+    "xs = c(", paste(xs, collapse = ", "),
+    "), p = c(", paste(p, collapse = ", "),
+    "), v = c(", paste(v, collapse = ", "),
+    "), centr_v = c(", paste(centr_v, collapse = ", "),
+    "), centr_v_weight = ", paste(centr_v_weight, collapse = ", "),
+    ", edge_weight = ", paste(edge_weight, collapse = ", "),
+    sep = ""
+  )
+  if (sum(is.na(c(p, v, centr_v, centr_v_weight, edge_weight))) > 0) {
+    stop(paste("There are NAs.", inval_input_message))
+  }
+  if (length(p) != 3 || length(v) != 3 || length(centr_v) != 3 ||
+    length(centr_v_weight) != 1 || length(edge_weight) != 1) {
+    stop(paste("The input has not the correct length.", inval_input_message))
+  }
+  if (min(xs) < 0 || max(xs) >= 1) {
+    stop(paste("xs out of range (0 to 1).", inval_input_message))
+  }
+  if (edge_weight < 0 || centr_v_weight < 0) {
+    stop(paste("Negative weights.", inval_input_message))
+  }
+  if (sum(p == v) == 3) {
+    warning(paste(
+      "Parent and child have the same coordinates.",
+      "Returning zero instead.", inval_input_message
+    ))
+    return(rep(0, length(xs)))
+  }
+  if (edge_weight == 0 && centr_v_weight == 0) {
+    warning(paste(
+      "All weights are zero. Returning zero.",
+      inval_input_message
+    ))
+    return(rep(0, length(xs)))
   }
   node_imbal_vals <- NULL
   if (imbal_type == "mu") {
@@ -602,18 +663,15 @@ imbalProfile_e <- function(xs, p, v, centr_v, centr_v_weight, edge_weight,
     })
   } else {
     stop(paste("Unknown node imbalance measurement '", imbal_type,
-               "'. Available are: 'A', 'alpha', 'M' and 'mu'.",
-               sep = ""))
+      "'. Available are: 'A', 'alpha', 'M' and 'mu'.",
+      sep = ""
+    ))
   }
   if (sum(is.na(node_imbal_vals)) > 0) {
-    stop(paste("A problem occured when trying to integrate using this input",
-               ": p=", paste(p, collapse = " "),
-               "; v=", paste(v, collapse = " "),
-               "; centr_v=", paste(centr_v, collapse = " "),
-               "; centr_v_weight=", paste(centr_v_weight, collapse = " "),
-               "; edge_weight=", paste(edge_weight, collapse = " "),
-               "; imbal_type=", paste(imbal_type, collapse = " "),
-               sep = ""))
+    stop(paste("A problem occured when trying to integrate the following ",
+      inval_input_message,
+      sep = ""
+    ))
   } else {
     return(node_imbal_vals)
   }
